@@ -6,7 +6,7 @@
 use serde::Deserialize;
 use serde::Serialize;
 
-use super::super::ManagedConnectorStatus;
+use super::super::{ManagedConnectorStatus, SecretString};
 use crate::k8_types::{Spec, Crd, DefaultHeader};
 use std::collections::BTreeMap;
 
@@ -38,12 +38,12 @@ impl Spec for K8ManagedConnectorSpec {
 #[serde(rename_all = "camelCase", default)]
 pub struct K8ManagedConnectorSpec {
     pub name: String,
-    pub connector_version: Option<String>,
+    pub version: Option<String>,
     #[cfg_attr(feature = "use_serde", serde(rename = "type"))]
     pub type_: String, // syslog, github star, slack
     pub topic: String,
     pub parameters: BTreeMap<String, String>,
-    pub secrets: BTreeMap<String, String>,
+    pub secrets: BTreeMap<String, SecretString>,
 }
 mod convert {
 
@@ -59,7 +59,7 @@ mod convert {
                 topic: spec.topic,
                 parameters: spec.parameters,
                 secrets: spec.secrets,
-                connector_version: spec.connector_version,
+                version: spec.version,
             }
         }
     }
@@ -72,7 +72,7 @@ mod convert {
                 topic: spec.topic,
                 parameters: spec.parameters,
                 secrets: spec.secrets,
-                connector_version: spec.connector_version,
+                version: spec.version,
             }
         }
     }
